@@ -1,4 +1,4 @@
-# Copyright (c) 2018-2022 Micro Focus or one of its affiliates.
+# Copyright (c) 2018-2023 Micro Focus or one of its affiliates.
 # Copyright (c) 2018 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -42,22 +42,17 @@ import socket
 import ssl
 import getpass
 import uuid
+import warnings
 from struct import unpack
 from collections import deque, namedtuple
 import random
 
 # noinspection PyCompatibility,PyUnresolvedReferences
-from six import raise_from, string_types, integer_types, PY2
-
-if PY2:
-    from urlparse import urlparse, parse_qs
-else:
-    from urllib.parse import urlparse, parse_qs
-
-    from typing import TYPE_CHECKING
-    if TYPE_CHECKING:
-        from typing import Any, Dict, Literal, Optional, Type, Union
-        from typing_extensions import Self
+from urllib.parse import urlparse, parse_qs
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    from typing import Any, Dict, Literal, Optional, Type, Union
+    from typing_extensions import Self
 
 import vertica_python
 from .. import errors
@@ -165,7 +160,7 @@ class _AddressList(object):
         # a host name or IP address string (using default port) or
         # a (host, port) tuple
         for node in backup_nodes:
-            if isinstance(node, string_types):
+            if isinstance(node, str):
                 self._append(node, DEFAULT_PORT)
             elif isinstance(node, tuple) and len(node) == 2:
                 self._append(node[0], node[1])
@@ -177,16 +172,16 @@ class _AddressList(object):
         self._logger.debug('Address list: {0}'.format(list(self.address_deque)))
 
     def _append(self, host, port):
-        if not isinstance(host, string_types):
+        if not isinstance(host, str):
             err_msg = 'Host must be a string: invalid value: {0}'.format(host)
             self._logger.error(err_msg)
             raise TypeError(err_msg)
 
-        if not isinstance(port, (string_types, integer_types)):
+        if not isinstance(port, (str, int)):
             err_msg = 'Port must be an integer or a string: invalid value: {0}'.format(port)
             self._logger.error(err_msg)
             raise TypeError(err_msg)
-        elif isinstance(port, string_types):
+        elif isinstance(port, str):
             try:
                 port = int(port)
             except ValueError as e:
@@ -453,21 +448,27 @@ class Connection(object):
         if self.socket:
             return self.socket
 
-        # the initial establishment of the client connection
+        # the initial establishment of the socket connection
         raw_socket = self.establish_socket_connection(self.address_list)
 
-        # enable load balancing
-        load_balance_options = self.options.get('connection_load_balance')
-        self._logger.debug('Connection load balance option is {0}'.format(
-                     'enabled' if load_balance_options else 'disabled'))
-        if load_balance_options:
-            raw_socket = self.balance_load(raw_socket)
+        # modify the socket connection based on client connection options
+        try:
+            # enable load balancing
+            load_balance_options = self.options.get('connection_load_balance')
+            self._logger.debug('Connection load balance option is {0}'.format(
+                         'enabled' if load_balance_options else 'disabled'))
+            if load_balance_options:
+                raw_socket = self.balance_load(raw_socket)
 
-        # enable SSL
-        ssl_options = self.options.get('ssl')
-        self._logger.debug('SSL option is {0}'.format('enabled' if ssl_options else 'disabled'))
-        if ssl_options:
-            raw_socket = self.enable_ssl(raw_socket, ssl_options)
+            # enable SSL
+            ssl_options = self.options.get('ssl')
+            self._logger.debug('SSL option is {0}'.format('enabled' if ssl_options else 'disabled'))
+            if ssl_options:
+                raw_socket = self.enable_ssl(raw_socket, ssl_options)
+        except:
+            self._logger.debug('Close the socket')
+            raw_socket.close()
+            raise
 
         self.socket = raw_socket
         return self.socket
@@ -518,7 +519,9 @@ class Connection(object):
             raw_socket = self.establish_socket_connection(self.address_list)
         else:
             self._logger.debug('<= LoadBalanceResponse: %s', response)
-            self._logger.warning("Load balancing requested but not supported by server")
+            no_load_balancing = "Load balancing requested but not supported by server"
+            warnings.warn(no_load_balancing)
+            self._logger.warning(no_load_balancing)
 
         return raw_socket
 
@@ -541,9 +544,9 @@ class Connection(object):
                 else:
                     raw_socket = ssl.wrap_socket(raw_socket)
             except ssl.CertificateError as e:
-                raise_from(errors.ConnectionError(str(e)), e)
+                raise errors.ConnectionError(str(e))
             except ssl.SSLError as e:
-                raise_from(errors.ConnectionError(str(e)), e)
+                raise errors.ConnectionError(str(e))
         else:
             err_msg = "SSL requested but not supported by server"
             self._logger.error(err_msg)
@@ -611,11 +614,12 @@ class Connection(object):
             self.close_socket()
             self._logger.error(str(e))
             if isinstance(e, IOError):
-                raise_from(errors.ConnectionError(str(e)), e)
+                raise errors.ConnectionError(str(e))
             else:
                 raise
 
     def close_socket(self):
+        self._logger.debug("Close connection's socket")
         try:
             if self.socket is not None:
                 self._socket().close()
@@ -645,6 +649,8 @@ class Connection(object):
             if getattr(self, 'notice_handler', None) is not None:
                 self.notice_handler(message)
             else:
+                notice = f'[{message.severity}] {message.message}'
+                warnings.warn(notice)
                 self._logger.warning(message.error_message())
 
     def read_string(self):
@@ -695,7 +701,7 @@ class Connection(object):
                 self.close_socket()
                 # noinspection PyTypeChecker
                 self._logger.error(e)
-                raise_from(errors.ConnectionError(str(e)), e)
+                raise errors.ConnectionError(str(e))
             if not self.is_asynchronous_message(message):
                 break
         return message
@@ -832,8 +838,9 @@ class Connection(object):
                     self._logger.error(msg)
                     raise errors.ConnectionError(msg)
                 elif message.code == messages.Authentication.PASSWORD_GRACE:
-                    self._logger.warning('The password for user {} will expire soon.'
-                        ' Please consider changing it.'.format(self.options['user']))
+                    password_grace = f'The password for user {self.options["user"]} will expire soon. Please consider changing it.'
+                    warnings.warn(password_grace)
+                    self._logger.warning(password_grace)
                 elif message.code == messages.Authentication.GSS:
                     self.make_GSS_authentication()
                 else:
