@@ -1,4 +1,4 @@
-# Copyright (c) 2018-2022 Micro Focus or one of its affiliates.
+# Copyright (c) 2018-2023 Micro Focus or one of its affiliates.
 # Copyright (c) 2018 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -43,7 +43,8 @@ import re
 import sys
 import traceback
 from decimal import Decimal
-from io import IOBase
+from io import IOBase, BytesIO, StringIO
+from math import isnan
 from tempfile import NamedTemporaryFile, SpooledTemporaryFile, TemporaryFile
 from uuid import UUID
 from collections import OrderedDict
@@ -55,20 +56,13 @@ try:
 except ImportError:
     _TemporaryFileWrapper = None
 
-import six
-# noinspection PyUnresolvedReferences,PyCompatibility
-from six import binary_type, text_type, string_types, integer_types, BytesIO, StringIO
-from six.moves import zip
-
-if six.PY3:
-    from typing import TYPE_CHECKING
-
-    if TYPE_CHECKING:
-        from typing import IO, Any, AnyStr, Callable, Dict, Generator, List, Literal, Optional, Sequence, Tuple, Type, TypeVar, Union
-        from typing_extensions import Self
-        from .connection import Connection
-        from logging import Logger
-        T = TypeVar('T')
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    from typing import IO, Any, AnyStr, Callable, Dict, Generator, List, Literal, Optional, Sequence, Tuple, Type, TypeVar, Union
+    from typing_extensions import Self
+    from .connection import Connection
+    from logging import Logger
+    T = TypeVar('T')
 
 from .. import errors, os_utils
 from ..compat import as_text
@@ -126,9 +120,6 @@ file_type = tuple(
     ]
     if inspect.isclass(type_)
 )
-if six.PY2:
-    # noinspection PyUnresolvedReferences
-    file_type = file_type + (file,)
 
 
 RE_NAME_BASE = u"[0-9a-zA-Z_][\\w\\d\\$_]*"
@@ -179,6 +170,22 @@ class Cursor(object):
     def __exit__(self, type_, value, traceback):
         self.close()
 
+    #############################################
+    # decorators
+    #############################################
+    def handle_ctrl_c(func):
+        """
+        On Ctrl-C, try to cancel the query in the server
+        """
+        def wrap(self, *args, **kwargs):
+            try:
+                return func(self, *args, **kwargs)
+            except KeyboardInterrupt:
+                self.connection.cancel()
+                self.flush_to_query_ready() # ignore errors.QueryCanceled
+                raise
+        return wrap
+
     #############################################
     # dbapi methods
     #############################################
@@ -192,6 +199,7 @@ class Cursor(object):
             self._close_prepared_statement()
         self._closed = True
 
+    @handle_ctrl_c
     def execute(self, operation, parameters=None, use_prepared_statements=None,
                 copy_stdin=None, buffer_size=DEFAULT_BUFFER_SIZE):
         # type: (str, Optional[Union[List[Any], Tuple[Any], Dict[str, Any]]], Optional[bool], Optional[Union[IO[AnyStr], List[IO[AnyStr]]]], int) -> Self
@@ -220,9 +228,17 @@ class Cursor(object):
         use_prepared = bool(self.connection.options['use_prepared_statements']
                 if use_prepared_statements is None else use_prepared_statements)
         if use_prepared:
+            #################################################################
             # Execute the SQL as prepared statement (server-side bindings)
-            if parameters and not isinstance(parameters, (list, tuple)):
-                raise TypeError("Execute parameters should be a list/tuple")
+            #################################################################
+            if parameters is not None:
+                if not isinstance(parameters, (list, tuple)):
+                    raise TypeError("Execute parameters should be a list/tuple")
+                elif parameters and '?' not in operation:
+                    raise ValueError(f'Invalid SQL: {operation}'
+                        '\nHINT: When use_prepared_statements=True, variables in SQL should be specified with '
+                        'question mark (?) placeholders. Positional format (%s) placeholders have to be used '
+                        'with use_prepared_statements=False setting.')
 
             # If the SQL has not been prepared, prepare the SQL
             if operation != self.prepared_sql:
@@ -232,13 +248,16 @@ class Cursor(object):
             # Bind the parameters and execute
             self._execute_prepared_statement([parameters])
         else:
+            #################################################################
             # Execute the SQL directly (client-side bindings)
+            #################################################################
             if parameters:
                 operation = self.format_operation_with_parameters(operation, parameters)
             self._execute_simple_query(operation)
 
         return self
 
+    @handle_ctrl_c
     def executemany(self, operation, seq_of_parameters, use_prepared_statements=None):
         # type: (str, Sequence[Union[List[Any], Tuple[Any], Dict[str, Any]]], Optional[bool]) -> None
 
@@ -257,7 +276,9 @@ class Cursor(object):
                 if use_prepared_statements is None else use_prepared_statements)
 
         if use_prepared:
+            #################################################################
             # Execute the SQL as prepared statement (server-side bindings)
+            #################################################################
             if len(seq_of_parameters) == 0:
                 raise ValueError("seq_of_parameters should not be empty")
             if not all(isinstance(elem, (list, tuple)) for elem in seq_of_parameters):
@@ -270,6 +291,9 @@ class Cursor(object):
             # Bind the parameters and execute
             self._execute_prepared_statement(seq_of_parameters)
         else:
+            #################################################################
+            # Rewrite the INSERT SQL into a COPY statement
+            #################################################################
             m = self._insert_statement.match(operation)
             if m:
                 target = as_text(m.group('target'))
@@ -442,9 +466,9 @@ class Cursor(object):
 
         self.flush_to_query_ready()
 
-        if isinstance(data, binary_type):
+        if isinstance(data, bytes):
             stream = BytesIO(data)
-        elif isinstance(data, text_type):
+        elif isinstance(data, str):
             stream = StringIO(data)
         elif isinstance(data, file_type) or callable(getattr(data, 'read', None)):
             stream = data
@@ -570,7 +594,7 @@ class Cursor(object):
         if type(py_obj) in self._sql_literal_adapters and not is_copy_data:
             adapter = self._sql_literal_adapters[type(py_obj)]
             result = adapter(py_obj)
-            if not isinstance(result, (string_types, bytes)):
+            if not isinstance(result, (str, bytes)):
                 raise TypeError("Unexpected return type of {} adapter: {}, expected a string type."
                     .format(type(py_obj), type(result)))
             return as_text(result)
@@ -579,15 +603,41 @@ class Cursor(object):
             return '' if is_copy_data else 'NULL'
         elif isinstance(py_obj, bool):
             return str(py_obj)
-        elif isinstance(py_obj, (string_types, bytes)):
+        elif isinstance(py_obj, (str, bytes)):
             return self.format_quote(as_text(py_obj), is_copy_data)
-        elif isinstance(py_obj, (integer_types, float, Decimal)):
+        elif isinstance(py_obj, (int, Decimal)):
+            return str(py_obj)
+        elif isinstance(py_obj, float):
+            if py_obj in (float('Inf'), float('-Inf')) or isnan(py_obj):
+                return f"'{str(py_obj)}'::FLOAT"
             return str(py_obj)
         elif isinstance(py_obj, tuple):  # tuple and namedtuple
             elements = [None] * len(py_obj)
             for i in range(len(py_obj)):
                 elements[i] = self.object_to_string(py_obj[i], is_copy_data)
             return "(" + ",".join(elements) + ")"
+        elif isinstance(py_obj, list) and not is_copy_data:
+            elements = [None] * len(py_obj)
+            for i in range(len(py_obj)):
+                elements[i] = self.object_to_string(py_obj[i], False)
+            # Use the ARRAY keyword to construct an array value
+            return f'ARRAY[{",".join(elements)}]'
+        elif isinstance(py_obj, set) and not is_copy_data:
+            elements = [None] * len(py_obj)
+            i = 0
+            for o in py_obj:
+                elements[i] = self.object_to_string(o, False)
+                i += 1
+            # Use the SET keyword to construct a set value
+            return f'SET[{",".join(elements)}]'
+        elif isinstance(py_obj, dict) and not is_copy_data:
+            elements = [None] * len(py_obj)
+            i = 0
+            for k, v in py_obj.items():
+                elements[i] = self.object_to_string(v, False) + f' AS "{k}"'
+                i += 1
+            # Use the ROW keyword to construct a row value
+            return f'ROW({",".join(elements)})'
         elif isinstance(py_obj, (datetime.datetime, datetime.date, datetime.time, UUID)):
             return self.format_quote(as_text(str(py_obj)), is_copy_data)
         else:
@@ -602,11 +652,13 @@ class Cursor(object):
 
     # noinspection PyArgumentList
     def format_operation_with_parameters(self, operation, parameters, is_copy_data=False):
-        operation = as_text(operation)
-
         if isinstance(parameters, dict):
-            for key, param in six.iteritems(parameters):
-                if not isinstance(key, string_types):
+            if parameters and ':' not in operation:
+                raise ValueError(f'Invalid SQL: {operation}'
+                    "\nHINT: When argument 'parameters' is a dict, variables in SQL should be specified with named (:name) placeholders."
+                    " If you use a dict to represent the value of a ROW type column, enclose the dict with brackets('[]') to construct a list.")
+            for key, param in parameters.items():
+                if not isinstance(key, str):
                     key = str(key)
                 key = as_text(key)
 
@@ -618,11 +670,15 @@ class Cursor(object):
                 operation = re.sub(match_str, lambda _: value, operation, flags=re.U)
 
         elif isinstance(parameters, (tuple, list)):
+            if parameters and '%s' not in operation:
+                raise ValueError(f'Invalid SQL: {operation}'
+                    "\nHINT: When argument 'parameters' is a tuple/list, "
+                    'variables in SQL should be specified with positional format (%s) placeholders. '
+                    'Question mark (?) placeholders have to be used with use_prepared_statements=True setting.')
             tlist = []
             for param in parameters:
                 value = self.object_to_string(param, is_copy_data)
                 tlist.append(value)
-
             operation = operation % tuple(tlist)
         else:
             raise TypeError("Argument 'parameters' must be dict or tuple/list")
@@ -924,3 +980,4 @@ class Cursor(object):
         self.connection.write(messages.Flush())
         self._message = self.connection.read_expected_message(messages.CloseComplete)
         self.connection.write(messages.Sync())
+
